const d = new Date();
const lastMonthDate = new Date(d.setMonth(d.getMonth() - 1));

const StateKeys = {
	CURRENT_USER: 'currentUser',
	USER_TO_MANAGE: 'userToManage',
	USER_TO_ADD: 'userToAdd',
	ROLE_TO_ADD: 'roleToEdit',
	REPORTS: 'reports',
	USERS: 'users',
	FILTERS: 'filters',
	DRAWERS: 'drawers',
	TAB_BARS: 'tabBars',
	APPSTATE: 'appState',
	TOAST_QUEUE: 'toastQueue',
	ENVIRONMENT: 'environment',
	STORE_SETUP: 'storeSetup',
	MANAGE_USERS: 'manageUsers'
};

const initialState = {
	users: {
		usersToAdd: [],
		userToAdd: {},
	},
	reports: {
		filter: {
			fromDate: DateUtils.formatDate(lastMonthDate),
			toDate: DateUtils.formatDate(new Date()),
		},
	},
	drawers: {},
	tabBars: {},
	environment: {},
	currentUser: {
		init: false,
	},
	manageUsers: {
		users: []
	}
};

let currentState;

const getInitialState = async () => {
	return new Promise(resolve => {
		setTimeout(() => {
			currentState =
				JSON.parse(localStorage.getItem(StateKeys.APPSTATE)) || initialState;
			resolve();
		}, 0);
	});
};

const State = {
	setState: async newState => {
		currentState = { ...currentState, ...newState };
		await setLocalStorageData(StateKeys.APPSTATE, currentState);
	},
	getState: async key => {
		return key ? (await currentState[key]) || {} : { ...(await currentState) };
	},
	setUsersState: async newState => {
		const usersState = await State.getState(StateKeys.USERS);
		await State.setState({
			users: {
				...usersState,
				...newState,
			},
		});
	},
	setStoreSetupData: async newState => {
		const storeSetupState = await State.getState(StateKeys.STORE_SETUP);
		await State.setState({
			storeSetup: {
				...storeSetupState,
				...newState,
			},
		});
	},
	updateUserToAdd: async (newState = false, updateUsers = true) => {
		const { multipleAdd, usersToAdd = [] } = await State.getState(
			StateKeys.USERS
		);
		const userToAdd = await State.getSingleOrMultipleToAddUser();
		const updatedUser = {
			...userToAdd,
			...newState,
		};
		if (multipleAdd && updateUsers && !!updatedUser?.id) {
			const userExists = usersToAdd?.find(u => u.id === updatedUser.id);
			if (!!userExists) {
				usersToAdd.splice(usersToAdd.indexOf(userExists), 1);
			}
			usersToAdd[!!usersToAdd?.length ? 'unshift' : 'push'](updatedUser);
		}
		await State.setUsersState({
			userToAdd: updatedUser,
			usersToAdd,
		});
		await UserFlow.toggleMultipleAddSaveBtn();
	},
	removeUser: async (userName, update = true) => {
		const { usersToAdd = [] } = await State.getState(StateKeys.USERS);

		await State.setUsersState({
			userToAdd: null,
			usersToAdd: usersToAdd.filter(
				u =>
					UserFlow.getEmailFromUser({
						eMail: u.eMail,
						userPrincipalName: u.userPrincipalName,
					}) !== userName
			),
		});
		if (update) {
			await DomHelper.updateAddMultipleUsersDrawer();
		}
	},
	getSingleOrMultipleToAddUser: async (userInState = StateKeys.USER_TO_ADD, clearRole = false) => {
		const {
			userToAdd,
			userToManage,
			multipleAdd = false,
			usersToAdd = [],
		} = await State.getState(StateKeys.USERS);

		const userToReturn = userInState === StateKeys.USER_TO_ADD ? userToAdd : userToManage;

		const isMultipleAddAndUsers = multipleAdd && !!usersToAdd.length;

		return !!userToReturn
			? userToReturn
			: isMultipleAddAndUsers
			? clearRole
				? { ...usersToAdd[0], role: null }
				: usersToAdd[0]
			: {};
	},
	setDrawerState: async (drawerId, drawer) => {
		if (!drawer) return;

		const drawers = await State.getState(StateKeys.DRAWERS);
		await State.setState({
			drawers: {
				...drawers,
				[drawerId]: drawer,
			},
		});
	},
	updateLoggedInUser: async data => {
		const loggedInUser = await State.getState(StateKeys.CURRENT_USER);
		await State.setState({
			currentUser: {
				...loggedInUser,
				...data,
			},
		});
	},
	setManageUsersState: async (usersToManage, replace = false) => {
		if (!usersToManage) return;

		let newUsersList = usersToManage;

		if(!replace){
			const {users = []} = await State.getState(StateKeys.MANAGE_USERS);
			if(users.length > 1){
				const prevUsersAndNewUsers = [...users, ...usersToManage]
				newUsersList = prevUsersAndNewUsers.filter((user,index) => prevUsersAndNewUsers.findIndex(uniqueUser => uniqueUser.userPrincipalName === user.userPrincipalName) === index);
			}
		}

		await State.setState({
			manageUsers: {
				users: newUsersList
			},
		});
	},
	getDrawerById: async drawerId => {
		const drawers = await State.getState(StateKeys.DRAWERS);
		const drawer = drawers[drawerId];
		return !!drawer ? drawer : null;
	},
	setFilterDateState: async (fromDate, toDate) => {
		const reports = await State.getState(StateKeys.REPORTS);
		const filter = reports?.filter;
		await State.setState({
			reports: {
				...reports,
				filter: {
					...filter,
					fromDate,
					toDate,
				},
			},
		});
	},
	extractDataStateValue: async dataAttributes => {
		const dataValues = {};

		for (const attributeName in dataAttributes) {
			if (dataAttributes.hasOwnProperty(attributeName)) {
				if (attributeName.startsWith('statetoadd')) {
					const attributeParts = attributeName
						.replace(/^statetoadd/, '')
						.match(/[A-Z][a-z]+/g);
					if (attributeParts && attributeParts.length >= 2) {
						const key = attributeParts.join('-');
						dataValues[key] = dataAttributes[attributeName];
					}
				}
			}
		}

		for (const key in dataValues) {
			if (dataValues.hasOwnProperty(key)) {
				const primaryKey = key.split('-')[0];
				const subKey = key.split('-')[1];
				const val = JSON.parse(dataValues[key]);
				await State.addDataAttrToState(primaryKey, subKey, val);
			}
		}
	},
	addDataAttrToState: async (primaryKey, subKey, dataVal) => {
		const users = await State.getState(StateKeys.USERS);

		if (!users) {
			console.error('Users state is empty or not defined.');
			return;
		}

		const stateKey = Object.values(StateKeys).find(
			sk => sk.toLocaleLowerCase() === primaryKey.toLocaleLowerCase()
		);
		if (!stateKey) return;

		const userObjInState = users[stateKey];
		if (!userObjInState) {
			console.error(`Users with primaryKey ${primaryKey} not found.`);
			return;
		}

		const userKeyExist = Object.values(StateKeys).find(
			sk => sk.toLocaleLowerCase() === subKey.toLocaleLowerCase()
		);
		const userKey = userKeyExist ? userKeyExist : subKey.toLowerCase();

		// Check if userKey is an object and parse it if necessary
		const actualUserKey =
			typeof userKey === 'object' ? JSON.stringify(userKey) : userKey;

		userObjInState[actualUserKey ? actualUserKey : subKey.toLowerCase()] =
			dataVal;
		await State.setUsersState(users);
	},
};

const setLocalStorageData = async (key, value) => {
	return new Promise(resolve => {
		setTimeout(() => {
			localStorage.setItem(key, JSON.stringify(value));
			resolve();
		}, 0);
	});
};

const getLocalStorageData = async key => {
	return new Promise(resolve => {
		setTimeout(() => {
			const data = localStorage.getItem(key);
			resolve(JSON.parse(data));
		}, 0);
	});
};

(async () => {
	await getInitialState();
})();
